IDLE_COMMAND = 0
BLINK_COMMAND = 1
VOLTAGE_COMMAND = 2
MPP_COMMAND = 3
READ_COMMAND = 4
CURVE_COMMAND = 5
TRANSFER_CURVE_COMMAND = 6
EXTERNAL_MPP_COMMAND = 7
TRIGGERED_CURVE_COMMAND = 8
GET_STATUS = 13
WRITE_EEPROM_COMMAND = 14
SET_TRIGGER_COMMAND = 15
READ_EEPROM_COMMAND = 16
UPDATE_CONFIG_COMMAND = 17
GET_CONFIG_COMMAND = 18
START_FIRMWARE_UPDATE = 19
ENABLE_FAST_COMMUNICATIONS = 20
DISABLE_BROADCAST = 21
SELF_RESET_CMD = 24
TRIGGERED_READ_COMMAND = 50
ALIVE = 100
LIST_MODULES = 101
OPEN_MODULE = 106
CLOSE_MODULE = 107
RESET_MODULE = 108
RESET_CONTROLLER = 109
TRIGGER_ALL = 110
BROADCAST_THRESHOLD_EXCEEDED = 111
ALIVE_RR1701 = 200
CURVE_RUNNING = 250
RR1700 = 0
RR1701 = 1


IVMPP_READ_IVPOINT = 20
IVMPP_APPLY_STATE = 21
PV_IRR_READ_IRRADIANCES = 30

VOC = 0
ISC = 1
MPP = 2
VOLTAGE_BIAS = 3

ISC_TO_VOC = 0
SWEEP_ISC_TO_VOC = 0
SWEEP_VOC_TO_ISC = 1
EXTENT_CURVE_DELAY = 2
SWEEP_VOC_ISC_VOC = 4
SWEEP_ISC_VOC_ISC = 8
